require 'metaverse/base'
require 'thor'

module Metaverse
  class Cli < Thor
    desc "init", "builds the repos folder structure"
    def init
      @meta = Base.new Dir.pwd
    end

    desc "reset", "Empties the repos cache"
    def reset
      init
      @meta.clear_repos
    end

    desc "branches", "Displays the branch in which each repo is"
    def branches
      init
      @meta.branches
    end

    desc "snapshot [COMMAND] [SNAPSHOT_NAME]", "Creates or loads a snapshot of the system"
    def snapshot command, name, remote = nil
      init
      case command
        when 'new' then @meta.create_state 'snapshot', name
        when 'load' then @meta.load_state 'snapshot', name, remote
        when 'send' then @meta.send_state 'snapshot', name, remote
      end
    end

    desc "feature [COMMAND] [FEATURE_NAME]", "Creates a new feature branch or loads a previous one"
    option :branch, type: :boolean, aliases: '-b'
    def feature command, name, remote = nil
      init
      case command
        when 'new' then @meta.create_state 'feature', name
        when 'load' then @meta.load_state 'feature', name, remote, options[:branch]
        when 'send' then @meta.send_state 'feature', name, remote
        when 'close' then @meta.send_state 'feature', name, remote, true
      end
    end


    desc "release [COMMAND] [FEATURE_NAME]", "Creates a new release branch or loads a previous one"
    option :branch, type: :boolean, aliases: '-b'
    def release command, name, remote = nil
      init
      case command
        when 'new' then @meta.create_state 'release', name
        when 'load' then @meta.load_state 'release', name, remote, options[:branch]
        when 'send' then @meta.send_state 'release', name, remote
        when 'close' then @meta.send_state 'release', name, remote, true
      end
    end


    desc "bugfix [COMMAND] [FEATURE_NAME]", "Creates a new bugfix branch or loads a previous one"
    option :branch, type: :boolean, aliases: '-b'
    def bugfix command, name, remote = nil
      init
      case command
        when 'new' then @meta.create_state 'bugfix', name
        when 'load' then @meta.load_state 'bugfix', name, remote, options[:branch]
        when 'send' then @meta.send_state 'bugfix', name, remote
        when 'close' then @meta.send_state 'bugfix', name, remote, true
      end
    end

    desc "status", "Displays the status of the system"
    def status
      init
      puts "System is at #{@meta.system_state.first}"
      @meta.status
    end

    desc "update [REMOTE_NAME]", "Fetches the latest code corresponding to the current branch. Remote is optional"
    def update remote = nil
      init
      @meta.update remote
    end

    desc "develop", "Switch the system to develop branch"
    def develop
      init
      @meta.checkout 'develop'
    end

    desc "checkout [BRANCH_NAME]", "Switches the system to another branch"
    def checkout name
      init
      @meta.checkout name
    end

    desc "check [COMMAND]", "Runs various checks on the system's health"
    def check command
      init
      case command
      when 'dirtiness' then @meta.check_dirtiness
      when 'consistency' then @meta.check_consistency
      end
    end

    desc "pull", "Pulls the latest changes from the main origin"
    def pull
      init
      @meta.pull
    end

    desc "remote [COMMAND]", "manages repos remotes such as adding new ones"
    def remote command, name, url
      init
      case command
      when 'add' then @meta.add_remote name, url
      end
    end

    desc "exec [COMMAND]", "runs the specified shell command in every repo"
    def exec *command
      init
      @meta.exec ENV, command.join(' ')
    end
  end
end